'use strict'
const path = require('path')
const express = require('express')
const webpack = require('webpack')
const makeWebpackConfig = require('./make-webpack-config')
const makeStatsOption = require('./make-stats-option')

module.exports = options => {
  return new Promise(resolve => {
    const webpackConfig = makeWebpackConfig(options)

    const app = express()

    const port = options.port || 4000

    webpackConfig.entry.client = [
      `webpack-hot-middleware/client`,
      webpackConfig.entry.client
    ]
    webpackConfig.output.publicPath = '/'
    webpackConfig.plugins.push(
      new webpack.HotModuleReplacementPlugin()
    )

    const compiler = webpack(webpackConfig)

    const devMiddleWare = require('webpack-dev-middleware')(compiler, {
      publicPath: webpackConfig.output.publicPath,
      stats: makeStatsOption(options.quiet)
    })

    app.use(devMiddleWare)
    app.use(require('webpack-hot-middleware')(compiler))

    const mfs = devMiddleWare.fileSystem
    const file = path.join(webpackConfig.output.path, 'index.html')

    devMiddleWare.waitUntilValid(resolve)

    app.get('*', (req, res) => {
      devMiddleWare.waitUntilValid(() => {
        const html = mfs.readFileSync(file)
        res.end(html)
      })
    })

    app.listen(port)
  })
}
