'use strict'
const path = require('path')
const webpack = require('webpack')
const ProgressPlugin = require('progress-bar-webpack-plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const paths = require('./paths')

module.exports = options => {
  const dev = options.dev
  const clientEntry = options.entry || paths.appEntry
  const dest = options.dest ? path.resolve(options.dest) : paths.appDist
  const template = options.template ? path.resolve(options.template) : paths.templatePath
  const devtool = dev ? 'cheap-module-source-map' : 'source-map'
  const env = dev ? 'development' : 'production'
  const filename = dev ? '[name].js' : '[name].[chunkhash:8].js'

  const config = {
    devtool,
    pathinfo: dev,
    entry: {
      client: clientEntry
    },
    output: {
      path: dest,
      filename,
      publicPath: './'
    },
    resolve: {
      extensions: ['.js', '.vue', '.css'],
      modules: paths.resolvePath
    },
    resolveLoader: {
      modules: paths.resolveLoaderPath
    },
    module: {
      loaders: [
        {
          test: /\.js$/,
          loaders: ['babel'],
          exclude: [/node_modules/]
        },
        {
          test: /\.vue$/,
          loaders: ['vue']
        },
        // file-loader makes sure that static files are dumped in ./dist folder
        // when you require it you get the filename
        {
          test: /\.(ico|jpg|png|gif|eot|otf|webp|svg|ttf|woff|woff2)(\?.*)?$/,
          loader: 'file',
          query: {
            name: 'static/media/[name].[hash:8].[ext]'
          }
        }
      ]
    },
    plugins: [
      new HtmlWebpackPlugin({
        template
      }),
      new webpack.DefinePlugin({
        'process.env.NODE_ENV': JSON.stringify(env)
      })
    ],
    babel: {
      presets: [
        [
          require.resolve('babel-preset-es2015'),
          {modules: false}
        ],
        require.resolve('babel-preset-stage-1')
      ],
      plugins: [
        require.resolve('babel-plugin-transform-vue-jsx')
      ]
    },
    postcss: [
      require('postcss-nested')
    ]
  }

  if (!dev) {
    if (!options.quiet) config.plugins.push(new ProgressPlugin())

    config.plugins.push(
      new webpack.LoaderOptionsPlugin({
        minimize: true
      }),
      new webpack.optimize.UglifyJsPlugin({
        sourceMap: true,
        compress: {
          warnings: false
        },
        output: {
          comments: false
        }
      })
    )
  }

  return config
}
